package node

import (
	"context"
	"fmt"
	"log"
	"os"
	"testing"
	"time"

	pb "git.autistici.org/ale/autoradio/proto"
	"go.etcd.io/etcd/embed"
	"go.etcd.io/etcd/etcdserver/api/v3client"
)

type fakeIcecast struct{}

func (f *fakeIcecast) GetStatus() ([]*pb.IcecastMount, bool) {
	return nil, true
}

func (f *fakeIcecast) Update(_ context.Context, mounts []*pb.Mount, isMaster bool, masterAddr string) error {
	return nil
}

func TestNode(t *testing.T) {
	cfg := embed.NewConfig()
	cfg.Dir = "default.etcd"
	defer os.RemoveAll(cfg.Dir)
	e, err := embed.StartEtcd(cfg)
	if err != nil {
		t.Fatalf("StartEtcd: %v", err)
	}
	<-e.Server.ReadyNotify()

	cli := v3client.New(e.Server)

	var nodes []*Node

	for i := 0; i < 2; i++ {
		n, err := New(
			cli,
			fmt.Sprintf("node%d", i+1),
			fmt.Sprintf("127.0.0.1:%d", 3000+i),
			fmt.Sprintf("127.0.0.1:%d", 4000+i),
			&fakeIcecast{},
			"random",
		)
		if err != nil {
			t.Fatalf("NewNode: %v", err)
		}

		go func() {
			time.Sleep(10 * time.Second)
			log.Printf("stopping everything")
			n.Stop()
		}()
		nodes = append(nodes, n)
	}

	for _, n := range nodes {
		n.Wait()
	}
}
